set rev(ip_cmp.tcl) {$Header: /home/cvs/PacketMaker/icmp.tcl,v 1.3 2003/03/23 04:37:09 clif Exp $}
#####################################################################
;# NAME:   ip_cmp.tcl
;# AUTHOR: Clif Flynt
;# DATE:   11/2002
;# DESC:   Procedures for building ICMP packets
;#         
;#         
;#
;# PARAMETERS:  
;#
;#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2002
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################
package provide icmp 1.0

namespace eval icmp {
    variable fields
    namespace export genPacket ICMP_ADDRESS ICMP_ADDRESSREPLY \
        ICMP_TIMESTAMPREPLY ICMP_TIMESTAMP ICMP_DEST_UNREACH
    
    proc genPacket {} {
      upvar fieldArray dataArray

      variable fields
      variable lookup

      set byteStream [$dataArray(type) dataArray]

      set p [lsearch $fields checksum]
      set pos [expr ([lindex $fields [incr p] ] / 8)]
      set len [expr ([lindex $fields [incr p]] / 8) -1]

      set checksum [packet::CheckSum $byteStream ]

      set dataArray(packet) \
          [eval lreplace [list $byteStream] $pos [expr $pos + $len] $checksum]
    }
    
    proc ICMP_TIMESTAMPREPLY  {dataArrayName} {
	upvar $dataArrayName dataArray
        return [ICMP_TIMESTAMP dataArray]
    }

    proc ICMP_TIMESTAMP {dataArrayName} {
        variable fields
        variable lookup
	upvar $dataArrayName dataArray
	set tFields $fields
	
	lappend tFields identifier 32 16 sequence 48 16 \
	    originate 64 32 receive 96 32 transmit 128 32
	set byteStream [packet::MakePayload $tFields lookup dataArray]
	return $byteStream
    }

    proc ICMP_ADDRESS {dataArrayName} {
        variable fields
        variable lookup
	upvar $dataArrayName dataArray
	set tFields $fields
	
	lappend tFields identifier 32 16 sequence 48 16 subnet 64 32
	set byteStream [packet::MakePayload $tFields lookup dataArray]
	return $byteStream
    }

    proc ICMP_DEST_UNREACH {dataArrayName} {
        variable fields
        variable lookup
	upvar $dataArrayName dataArray
	set tFields $fields
	
	lappend tFields zero 32 32 ipHeader 64 160 payloadHeader 168 64
	set byteStream [packet::MakePayload $tFields lookup dataArray]
	return $byteStream
    }

    proc ICMP_ADDRESSREPLY {dataArrayName} {
        variable fields
        variable lookup
	upvar $dataArrayName dataArray
	set tFields $fields
	
	lappend tFields identifier 32 16 sequence 48 16 subnet 64 32
	set byteStream [packet::MakePayload $tFields lookup dataArray]
	return $byteStream
    }
}


