set rev(ip_def.tcl) {$Header: /home/cvs/PacketMaker/ip_def.tcl,v 1.1.1.1 2002/12/02 01:37:04 clif Exp $}
#####################################################################
;# NAME:   ip_def.tcl
;# AUTHOR: Clif Flynt
;# DATE:   11/2002
;# DESC:   Definitions for IP packets
;#         
;#         
;#
;# PARAMETERS:  
;#
;#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2002
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################

package provide ip 1.0
namespace eval ip {
  variable fields 

  set fields { 
      version     0  4
      hdrlen      4  4
      tos         8  8
      length     16 16
      id         32 16
      flag       48  3
      offset     51 13
      ttl        64  8
      protocol   72  8
      checksum   80 16
      source     96 32
      dest      128 32
      options   160  0
      payload   160  {[expr 8 * [llength $val]]}
  }

# Pseudo header fields for tcp/udp checksum calculation
set pFields {
    source	0	32
    dest	32	32
    zero	64	8
    protocol	72	8
    pLength	80	16
}

variable lookup
set lookup(protocol.ICMP) 			     1
set lookup(protocol.ICMP.def) 			{encapsulates ICMP packet}
set lookup(protocol.IGMP) 			     2
set lookup(protocol.IGMP.def) 			{encapsulates IGMP packet}
set lookup(protocol.AH) 			     51
set lookup(protocol.AH.def) 			{encapsulates AH packet}
set lookup(protocol.TCP) 			     6
set lookup(protocol.TCP.def) 			{encapsulates TCP packet}
set lookup(protocol.UDP) 			     17
set lookup(protocol.UDP.def) 			{encapsulates UDP packet}

set lookup(IP_RF)                                0x8000  
set lookup(IP_RF.def)                               {reserved fragment flag }
set lookup(IP_DF)                                0x4000  
set lookup(IP_DF.def)                               {dont fragment flag }
set lookup(IP_MF)                                0x2000  
set lookup(IP_MF.def)                               {more fragments flag }
set lookup(IP_OFFMASK)                           0x1fff  
set lookup(IP_OFFMASK.def)                          {mask for fragmenting bits }
set lookup(IPVERSION)                            4       
set lookup(IPVERSION.def)                           {IP version number }
set lookup(IP_MAXPACKET)                         65535   
set lookup(IP_MAXPACKET.def)                        {maximum packet size }
set lookup(IPTOS_TOS_MASK)                       0x1E    
set lookup(IPTOS_LOWDELAY)                       0x10    
set lookup(IPTOS_THROUGHPUT)                     0x08    
set lookup(IPTOS_RELIABILITY)                    0x04    
set lookup(IPTOS_LOWCOST)                        0x02    
set lookup(IPTOS_MINCOST)                        IPTOS_LOWCOST
set lookup(IPTOS_PREC_MASK)                      0xe0    
set lookup(IPTOS_PREC_NETCONTROL)                0xe0    
set lookup(IPTOS_PREC_INTERNETCONTROL)           0xc0    
set lookup(IPTOS_PREC_CRITIC_ECP)                0xa0    
set lookup(IPTOS_PREC_FLASHOVERRIDE)             0x80    
set lookup(IPTOS_PREC_FLASH)                     0x60    
set lookup(IPTOS_PREC_IMMEDIATE)                 0x40    
set lookup(IPTOS_PREC_PRIORITY)                  0x20    
set lookup(IPTOS_PREC_ROUTINE)                   0x00    
set lookup(IPOPT_COPY)                           0x80    
set lookup(IPOPT_CLASS_MASK)                     0x60    
set lookup(IPOPT_NUMBER_MASK)                    0x1f    
set lookup(IPOPT_CONTROL)                        0x00    
set lookup(IPOPT_RESERVED1)                      0x20    
set lookup(IPOPT_DEBMEAS)                        0x40    
set lookup(IPOPT_RESERVED2)                      0x60    
set lookup(IPOPT_EOL)                            0       
set lookup(IPOPT_EOL.def)                           {end of option list }
set lookup(IPOPT_NOP)                            1       
set lookup(IPOPT_NOP.def)                           {no operation }
set lookup(IPOPT_RR)                             7       
set lookup(IPOPT_RR.def)                            {record packet route }
set lookup(IPOPT_TS)                             68      
set lookup(IPOPT_TS.def)                            {timestamp }
set lookup(IPOPT_SECURITY)                       130     
set lookup(IPOPT_SECURITY.def)                      {provide s,c,h,tcc }
set lookup(IPOPT_LSRR)                           131     
set lookup(IPOPT_LSRR.def)                          {loose source route }
set lookup(IPOPT_SATID)                          136     
set lookup(IPOPT_SATID.def)                         {satnet id }
set lookup(IPOPT_SSRR)                           137     
set lookup(IPOPT_SSRR.def)                          {strict source route }
set lookup(IPOPT_RA)                             148     
set lookup(IPOPT_RA.def)                            {router alert }
set lookup(IPOPT_OPTVAL)                         0       
set lookup(IPOPT_OPTVAL.def)                        {option ID }
set lookup(IPOPT_OLEN)                           1       
set lookup(IPOPT_OLEN.def)                          {option length }
set lookup(IPOPT_OFFSET)                         2       
set lookup(IPOPT_OFFSET.def)                        {offset within option }
set lookup(IPOPT_MINOFF)                         4       
set lookup(IPOPT_MINOFF.def)                        {min value of above }
set lookup(MAX_IPOPTLEN)                         40      
set lookup(IPOPT_TS_TSONLY)                      0       
set lookup(IPOPT_TS_TSONLY.def)                     {timestamps only }
set lookup(IPOPT_TS_TSANDADDR)                   1       
set lookup(IPOPT_TS_TSANDADDR.def)                  {timestamps and addresses }
set lookup(IPOPT_TS_PRESPEC)                     3       
set lookup(IPOPT_TS_PRESPEC.def)                    {specified modules only }
set lookup(IPOPT_SECUR_UNCLASS)                  0x0000  
set lookup(IPOPT_SECUR_CONFID)                   0xf135  
set lookup(IPOPT_SECUR_EFTO)                     0x789a  
set lookup(IPOPT_SECUR_MMMM)                     0xbc4d  
set lookup(IPOPT_SECUR_RESTR)                    0xaf13  
set lookup(IPOPT_SECUR_SECRET)                   0xd788  
set lookup(IPOPT_SECUR_TOPSECRET)                0x6bc5  
set lookup(IPDEFTTL)                             64      
set lookup(IPDEFTTL.def)                            {default ttl, from RFC 1340 }
set lookup(IPFRAGTTL)                            60      
set lookup(IPFRAGTTL.def)                           {time to live for frags, slowhz }
set lookup(IPTTLDEC)                             1       
set lookup(IPTTLDEC.def)                            {subtracted when forwarding }
set lookup(IP_MSS)                               576     
set lookup(IP_MSS.def)                              {default maximum segment size }
}
