set rev(packet.tcl) {$Header: /home/cvs/PacketMaker/packet.tcl,v 1.4 2003/03/23 04:37:09 clif Exp $}
#####################################################################
;# NAME:   packet.tcl
;# AUTHOR: Clif Flynt
;# DATE:   12/2002
;# DESC:   Creates network packets suitable for dnet
;#
;# PARAMETERS:  
;#	Many and varied
;#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2002
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################

package require icmp
package require ip
package require tcp
package require udp
package require ether

package provide packet 1.0

    proc debugPuts {str} {
        puts $str
    }
    

namespace eval packet {
  namespace export length setField getField replaceField
        proc length {} {
            upvar fieldArray  fieldArray 
	    return [llength $fieldArray(packet)]
        }
        
        proc setField {fieldName data} {
            upvar fieldArray  fieldArray 
            set fieldArray($fieldName) $data
        }

        proc getField {fieldName} {
            upvar fieldArray  fieldArray 
            return $fieldArray($fieldName) 
        }

	proc replaceField {fieldName data} {
	    upvar fieldArray fieldArray
	    setField $fieldName $data
	    if {![string match $fieldName checksum]} {
	        # Must reset of checksum calculation will break.
	        setField checksum 0
	    }
	    uplevel genPacket
	}
	
  variable newObjectCmds {
        variable fieldArray   ;# An array of fieldArray(fieldName) value
  }
        
    
    variable unique 0

    proc make {type args} {
        variable unique
	variable newObjectCmds
        set id packet_[incr unique]
	namespace eval ::$id $newObjectCmds
        namespace eval ::$id [list array set fieldArray $args]
        namespace eval ::$id "set type ::[string tolower $type]"
# puts "ID: $id CHILD: [namespace children ::]"
        namespace eval ::$id {namespace import ${type}::*}
        namespace eval ::$id {namespace import ::packet::*}

        set cmd [format {namespace eval ::%s $args} $id]
        uplevel #0 [list proc $id {args} $cmd]
	$id genPacket
	return $id
	}

	
    proc MakePayload {fields tableName dataArrayName} {
        upvar $dataArrayName dataArray 
    
        foreach {name start len} $fields {
            if {![info exists dataArray($name)]} {
    	    foreach {n start len} $fields {lappend names $n}
    	    error "Parameter $name Missing"\
    	        "Missing Parameter: $name\n Parameter list must include: \n[lsort $names] \nhas \n([lsort [array names dataArray]])" 
    	        
    	}
            eval lappend payload [MakeEntry $dataArray($name) $name $len $tableName]
        }
        return [Bits2Bytes $payload]
    }
    
    proc MakeEntry {val name len valArrayName } {
    debugPuts "MAKEENTRY:val $val name $name len $len valArrayName $valArrayName -- [namespace current] "
        if {[string match "" $val ]} {return ""}
      
      # If the value is a packet identifier, extract the field from the
      #   named packet.

      if {![string match [info procs ::$val] ""]} {
	    set val [$val getField packet]
      }

        if {![string is integer [lindex $val 0]]} {
          if {[string match {*.*.*.*} $val]} {
    	    set val [split $val .]	  
    	  } elseif {[string match {*:*:*:*:*:*} $val]} {
    	    set vv [split $val :]
    	    set val ""
    	    foreach v $vv {
    	        lappend val 0x$v
    	    }
    	  } elseif {[catch {set val $valArray($val)}]} {
                upvar 2 $valArrayName valArray
		if {[catch {set val $valArray($val)}]} {
        	   set val $valArray($name.$val)
		}
    	    }
        }
        
        # debugPuts "MakeEntry $val -- $name -- $len -- $valArrayName"
    
       
       set len [expr $len / [llength $val]]
    
       #returns a bitslist, e.g. int2bits 10 => {1 0 1 0}
       # Each number left padded to len bits with 0s
    
       foreach v $val {
         set bits ""
         while {$v != 0} {
             set bits "[expr {$v % 2}]$bits"
             set v [expr {0x7fffffff & ($v >> 1)}]
         }
         if {$bits==""} {set bits 0}
         while {$len > [string length $bits]} {set bits "0$bits"}
         lappend bitStream $bits
       }
    
   #  debugPuts "Start: VAL: $val LEN: $len ([llength $bitStream]) END: $bitStream"
      return $bitStream
    }
    
    proc Bits2Bytes {bitStream} {
        array set lkup {
            0000 0 0001 1 0010 2 0011 3 0100 4 0101 5 0110 6 0111 7 
    	1000 8 1001 9 1010 a 1011 b 1100 c 1101 d 1110 e 1111 f}
    
        set bitStream [string map {{ } {}} $bitStream]
        regsub -all {....} $bitStream {\0 } hexWords
        
        foreach {h l} $hexWords {
            lappend bytes 0x$lkup($h)$lkup($l)
        }
        return $bytes
    }
    
    
    proc Hex2Bytes {data} {
      foreach d $data {
        if {[string length $d] & 0x01} {
           set d 0$d
        }
        regsub -all {..} [join $d ""] {\0 } bytes
        eval lappend bytestream $bytes
      }
      return $bytestream
    }
    
    proc CheckSum {data} {
       set tot 0;
       foreach {h l} $data {
          set tot [expr $tot + $l + (256 * $h)]
       }
            
       while {$tot > 0xffff} {
          set tot [expr ($tot & 0xffff) + ($tot >> 16)]
       }
       set tot [expr -$tot -1]
    
       set chksum [lrange [Hex2Bytes [format %04x $tot]] 2 end]
       
       foreach v $chksum {
           lappend rtn 0x$v
       }
       return $rtn
    }
    	
}    
