set rev(testGen.tcl) {$Header: /home/cvs/PacketMaker/testGen.tcl,v 1.3 2003/03/23 04:37:09 clif Exp $}
#####################################################################
;# NAME:   testGen.tcl
;# AUTHOR: Clif Flynt
;# DATE:   11/2002
;# DESC:   Generate packets and output them to eth1
;#         
;#         
;#
;# PARAMETERS:  
;#
;#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2002
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################

# pkg_mkIndex -verbose . ip.tcl icmp.tcl packet.tcl *def*tcl tcp.tcl dnet.tcl udp.tcl ether.tcl libdnet.so
set root [file dirname [info script]]
lappend auto_path [file dirname [info script]]
puts "AP: $auto_path"

package require packet
package require ip
package require icmp
package require ether
package require tcp
package require udp
package require dnetlib

package require dnet

set e [dnet::open eth1]

# Generate a good icmp Address Mask Request
set icmp1 [packet::make ICMP type ICMP_ADDRESS code 0 checksum 0 identifier 1 \
    sequence 2 subnet 00]

# Generate a bad checksum icmp Address Mask Request
set icmp2 [packet::make ICMP type ICMP_ADDRESS code 0 checksum 99 identifier 1 \
    sequence 2 subnet 00]

set TCPpayload {0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}


set tcp1 [packet::make TCP source 0x3bdf dest 0xa33f seq {0x8519 0x74c8} ack_seq 0x70142b97 \
     hlen 0x8 res 0 urg 0 ack 1 psh 1 rst 0 syn 0 fin 0 window 0x7d78 checksum 0\
     urg_ptr 0 option {} payload $TCPpayload]

set tcp2 [packet::make TCP source 0x3bdf dest 0xa33f seq {0x8519 0x74c8} ack_seq 0x70142b97 \
     hlen 0x8 res 0 urg 0 ack 1 psh 1 rst 0 syn 0 fin 0 window 0x7d78 checksum 99\
     urg_ptr 0 option {} payload $TCPpayload]


set ip1 [packet::make IP version 4 hdrlen 5 tos 0 length 32 id 2 flag 0 \
    offset 0 ttl 32 protocol ICMP checksum 0 source \
    192.168.9.2 dest 192.168.9.17 options {} payload [$icmp1 getField packet]]

set ip2 [packet::make IP version 4 hdrlen 5 tos 0 length 32 id 2 flag 0 \
    offset 0 ttl 32 protocol ICMP checksum 0 source \
    192.168.9.2 dest 192.168.9.17 options {} payload [$icmp2 getField packet]]

# Generate a good icmp TIMESTAMP request
set icmp1 [packet::make ICMP type ICMP_TIMESTAMP code 0 \
    checksum 0 identifier 1 \
    sequence 2 originate 0x98765430 receive 0x98765431 transmit 0x98765432]

set ip1 [packet::make IP version 4 hdrlen 5 tos 0 length 32 id 2 flag 0 \
    offset 0 ttl 32 protocol ICMP checksum 0 source \
    192.168.9.2 dest 192.168.9.17 options {} payload [$icmp1 getField packet]]

set pld1 [$ip1 getField packet]

# Make a good DESTINATION UNREACHABLE icmp message, using the 
#  previous IP Packet for the body.

set icmp2 [packet::make ICMP type ICMP_DEST_UNREACH \
     code ICMP_DEST_UNREACH.ICMP_NET_UNREACH \
     checksum 0 zero 0 ipHeader [lrange $pld1 0 19] \
     payloadHeader [lrange $pld1 20 27] ]

set pld  [$icmp2 getField packet]

# Generate a good ICMP_ADDRESS packet.
#set icmp1 [packet::make ICMP type ICMP_ADDRESS code 0 checksum 0 identifier 1 \
#    sequence 2 subnet 00]
#set pld  [$icmp1 getField packet]

set ip2 [packet::make IP version 4 hdrlen 5 tos 0 \
    length [expr 20 + [llength $pld]]  id 2 flag 0 \
    offset 0 ttl 32 protocol ICMP checksum 0 source \
    192.168.9.2 dest 192.168.9.17 options {} payload $pld]

set ip3 [packet::make IP version 4 hdrlen 5 tos 0 length [expr 20 + [llength $tcp1]]\
    id 0x2 flag 0 \
    offset 0 ttl 64 protocol TCP checksum 0 source \
    192.168.9.17 dest 192.168.9.2 options {} payload [$tcp1 getField packet]]

set ip4 [packet::make IP version 4 hdrlen 5 tos 0 length [expr 20 + [llength $tcp1]]\
    id 0x9a31  flag 2 \
    offset 0 ttl 64 protocol TCP checksum 0 source \
    192.168.9.17 dest 192.168.9.2 options {} payload [$tcp2 getField packet]]


set goodIP_UDP {0x45 0x00 0x00 0x3a 0xca 0xb1 0x00 0x00 0x40 0x11 0x70 0xeb 0x98 0xa0 0x0c 0xd5
0x98 0xa0 0x01 0x01 0x04 0x01 0x00 0x35 0x00 0x26 0xa3 0x42 0xeb 0xe7 0x01 0x00
0x00 0x01 0x00 0x00 0x00 0x00 0x00 0x00 0x03 0x77 0x77 0x77 0x04 0x62 0x73 0x64
0x69 0x03 0x63 0x6f 0x6d 0x00 0x00 0x01 0x00 0x01
}

set goodUDP {0x04 0x01 0x00 0x35 0x00 0x26 0xa3 0x42 0xeb 0xe7 0x01 0x00
0x00 0x01 0x00 0x00 0x00 0x00 0x00 0x00 0x03 0x77 0x77 0x77 0x04 0x62 0x73 0x64
0x69 0x03 0x63 0x6f 0x6d 0x00 0x00 0x01 0x00 0x01
}

set goodUDP_MTCS {0x04 0x01 0x00 0x35 0x00 0x26 0x00 0x00 0xeb 0xe7 0x01 0x00
0x00 0x01 0x00 0x00 0x00 0x00 0x00 0x00 0x03 0x77 0x77 0x77 0x04 0x62 0x73 0x64
0x69 0x03 0x63 0x6f 0x6d 0x00 0x00 0x01 0x00 0x01
}

set domainBODY {0xeb 0xe7 0x01 0x00
0x00 0x01 0x00 0x00 0x00 0x00 0x00 0x00 0x03 0x77 0x77 0x77 0x04 0x62 0x73 0x64
0x69 0x03 0x63 0x6f 0x6d 0x00 0x00 0x01 0x00 0x01
}

set ip5 [packet::make IP version 4 hdrlen 5 tos 0 length [expr 20 + [llength $goodUDP_MTCS]]\
    id 0xcab1  flag 0 \
    offset 0 ttl 64 protocol UDP checksum 0 source \
    152.160.12.213 dest 152.160.1.1 options {} payload $goodUDP_MTCS]

set udp1 [packet::make UDP source 1025 dest 53 length [expr [llength $domainBODY] + 8] \
    checksum 0 payload $domainBODY]

set udpBAD [packet::make UDP source 1025 dest 53 length [expr [llength $domainBODY] + 8] \
    checksum 9 payload $domainBODY]

set ip7 [packet::make IP version 4 hdrlen 5 tos 0 length [expr 20 + [llength [$udp1 getField packet]]]\
    id 0xcab1  flag 0 \
    offset 0 ttl 64 protocol UDP checksum 0 source \
    152.160.12.213 dest 152.160.1.1 options {} payload [$udp1 getField packet]]

 set ip8 [packet::make IP version 4 hdrlen 5 tos 0 length [expr 20 + [llength $udpBAD]]\
    id 0xcab1  flag 0 \
    offset 0 ttl 64 protocol UDP checksum 0 source \
    152.160.12.213 dest 152.160.1.1 options {} payload [$udpBAD getField packet]]


set ep1 [packet::make ETHER dest 00:E0:4C:00:14:4D src 00:A0:CC:D1:B6:00 \
    type IP payload [$ip1 getField packet]]
set ep2 [packet::make ETHER dest 00:E0:4C:00:14:4D src 00:A0:CC:D1:B6:00 \
    type IP payload [$ip2 getField packet]]
set ep3 [packet::make ETHER dest 00:E0:4C:00:14:4D src 00:A0:CC:D1:B6:00 \
    type IP payload [$ip3 getField packet]]
set ep4 [packet::make ETHER dest 00:E0:4C:00:14:4D src 00:A0:CC:D1:B6:00 \
    type IP payload [$ip4 getField packet]]
set ep5 [packet::make ETHER dest 00:E0:4C:00:14:4D src 00:A0:CC:D1:B6:00 \
    type IP payload [$ip5 getField packet]]
set ep7 [packet::make ETHER dest 00:E0:4C:00:14:4D src 00:A0:CC:D1:B6:00 \
    type IP payload [$ip7 getField packet]]
set ep8 [packet::make ETHER dest 00:E0:4C:00:14:4D src 00:A0:CC:D1:B6:00 \
    type IP payload [$ip8 getField packet]]

set goodIP {0x45 0x00 0x00 0x58 0x9a 0x31 0x40 0x00 0x40 0x06 0x0d 0x18 0xc0 0xa8 0x09 0x04\
0xc0 0xa8 0x09 0x02 0x3b 0xdf 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x80 0x18 0x7d 0x78 0x09 0x5f 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

puts "IP5: $ip5"
puts "GIP: $goodIP_UDP"

set j 0
foreach i [$ip5 getField packet] g $goodIP_UDP {
    if {$i != $g} {
        puts "MISMATCH at pos $j: $i != $g"
    }
    incr j
}

set ep6 [packet::make ETHER dest 00:E0:4C:00:14:4D src 00:A0:CC:D1:B6:00 type IP payload $goodIP]

foreach ep {ep1 ep2 ep3 ep4 ep5 ep6 ep7} {
     dnet::importPacket [set $ep]
}

puts "OK:  $ep1"
puts "BAD: $ep2"

# ep1 OK icmp
# ep2 ip checksum error icmp
# ep3 OK
# ep4 TCP checksum error
# ep5 UDP OK
# ep6 TCP/IP OK
# ep7 UDP - OK built from UDP
# ep8 UDP - UDP Checksum error

for {set i 0} {$i < 100} {incr i} {
#    dnet::send $e $ep1
    dnet::send $e $ep2
#    dnet::send $e $ep3
#    dnet::send $e $ep4
#    dnet::send $e $ep5
#    dnet::send $e $ep6
#    dnet::send $e $ep7
#    dnet::send $e $ep8
}


