set rev(testpm.tcl) {$Header: /home/cvs/PacketMaker/testpm.tcl,v 1.1.1.1 2002/12/02 01:37:04 clif Exp $}
#####################################################################
;# NAME:   testpm.tcl
;# AUTHOR: Clif Flynt
;# DATE:   11/2002
;# DESC:   Generate packets and compare them to predefined good 
;#         packets.
;#         
;#         
;#
;# PARAMETERS:  
;#
;#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2002
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################

pkg_mkIndex -verbose . ip.tcl icmp.tcl packet.tcl *def*tcl tcp.tcl dnet.tcl udp.tcl ether.tcl
lappend auto_path .

package require tcltest
package require packet

proc compareLists {one two} {
    set pos 0
    foreach a $one b $two {
    if {$a != $b} {
        puts "$pos $a $b"
    }
    incr pos
    }
}
# ::tcltest::test  name desc ?constraint? script expectedAnswer
tcltest::test MakeEntry-1 "test simple entry" {} {packet::MakeEntry 1.2.3.4 32 32 foo} {00000001 00000010 00000011 00000100}


set p1 [packet::make ICMP type ICMP_ADDRESS code 0 checksum 0 identifier 1 \
    sequence 2 subnet 00]

tcltest::test makeICMP-1 {ICMP_ADDRESS} {} \
    {$p1 getField packet} \
    {0x11 0x00 0xee 0xfc 0x00 0x01 0x00 0x02 0x00 0x00 0x00 0x00}

puts "$errorInfo"

set p2 [packet::make ICMP type ICMP_ADDRESS code 0 checksum 0 identifier 1 \
    sequence 0x1f subnet 00]

tcltest::test makeICMP-2 {ICMP_ADDRESS w/Hex} {} \
    {$p2 getField packet} \
    {0x11 0x00 0xee 0xdf 0x00 0x01 0x00 0x1f 0x00 0x00 0x00 0x00}


set pl1 [packet::make ICMP type ICMP_ADDRESSREPLY code 0 checksum 0 identifier 1 sequence 0x1f subnet 0x12345678]
set payload [$pl1 getField packet]

set p3 [packet::make ICMP type ICMP_ADDRESSREPLY code 0 checksum 0 identifier 1 \
    sequence 0x1f subnet 0x12345678]

tcltest::test makeICMP-2 {ICMP_ADDRESSREPLY} {} \
    {$p3 getField packet} \
    {0x12 0x00 0x85 0x33 0x00 0x01 0x00 0x1f 0x12 0x34 0x56 0x78}

set p4 [packet::make IP version 4 hdrlen 5 tos 0 length 0 id 2 flag 0 offset 0 ttl 32 \
    protocol ICMP checksum 0 source 192.168.9.2 dest 192.168.9.17 options {} payload $payload]

tcltest::test makeIP-1 {IP for ICMP Address Request} {} \
    {$p4 getField packet} \
    {0x45 0x00 0x00 0x00 0x00 0x02 0x00 0x00 0x20 0x01 0x07 0x98 0xc0 0xa8 0x09 0x02\
     0xc0 0xa8 0x09 0x11 0x12 0x00 0x85 0x33 0x00 0x01 0x00 0x1f 0x12 0x34 0x56 0x78}


set ipHdr {0x45 0x00 0x00 0x54 0x00 0x00 0x40 0x00 0x40 0x01 0x00 0x00 
    0xc0 0xa8 0x5a 0x40 0xc0 0xa8 0x5a 0x02} 

tcltest::test Checksum-1 {Checksum for small header} {} \
    {packet::CheckSum $ipHdr} \
    {0x05 0x16}


#      version     0  4 -- 4
#      hdrlen      4  4 -- 5
#      tos         8  8 -- 00
#      length     16 16 -- 00 54
#      id         32 16 -- 00 00
#      flag       48  3 --
#      offset     51 13 -- 4000
#      ttl        64  8 -- 40
#      protocol   72  8 -- 01
#      checksum   80 16 -- 00 00
#      source     96 32 -- c0 a8 5a 40 - 192.168.90.64
#      dest      128 32 -- c0 a8 5a 02 - 192.168.90.2
#      options   160  0



set icmpEchoRequestPayload { 0x08 0x00 0x98 0xd9 0xdf 0x22 0x00 0x00
0x63 0xcf 0x4d 0x3d 0xd5 0xf3 0x0e 0x00 0x08 0x09 0x0a 0x0b 0x0c 0x0d
0x0e 0x0f 0x10 0x11 0x12 0x13 0x14 0x15 0x16 0x17 0x18 0x19 0x1a 0x1b
0x1c 0x1d 0x1e 0x1f 0x20 0x21 0x22 0x23 0x24 0x25 0x26 0x27 0x28 0x29
0x2a 0x2b 0x2c 0x2d 0x2e 0x2f 0x30 0x31 0x32 0x33 0x34 0x35 0x36 0x37}

set p5 [packet::make IP version 4 hdrlen 5 tos 0 length 0x54 id 0 flag 2 offset 0 ttl 0x40 \
    protocol ICMP checksum 0 source 192.168.90.64 dest 192.168.90.2 options {} \
    payload $icmpEchoRequestPayload]

tcltest::test IP-1  {IP Packet with known body} {} \
    {$p5 getField packet} \
    {0x45 0x00 0x00 0x54 0x00 0x00 0x40 0x00 0x40 0x01 0x05 0x16 0xc0 0xa8\
     0x5a 0x40 0xc0 0xa8 0x5a 0x02 0x08 0x00 0x98 0xd9 0xdf 0x22 0x00 0x00\
     0x63 0xcf 0x4d 0x3d 0xd5 0xf3 0x0e 0x00 0x08 0x09 0x0a 0x0b 0x0c 0x0d\
     0x0e 0x0f 0x10 0x11 0x12 0x13 0x14 0x15 0x16 0x17 0x18 0x19 0x1a 0x1b\
     0x1c 0x1d 0x1e 0x1f 0x20 0x21 0x22 0x23 0x24 0x25 0x26 0x27 0x28 0x29\
     0x2a 0x2b 0x2c 0x2d 0x2e 0x2f 0x30 0x31 0x32 0x33 0x34 0x35 0x36 0x37}


set goodIP {0x45 0x00 0x00 0x58 0x9a 0x31 0x40 0x00 0x40 0x06 0x0d 0x18 0xc0 0xa8 0x09 0x04\
0xc0 0xa8 0x09 0x02 0x3b 0xdf 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x80 0x18 0x7d 0x78 0x09 0x5f 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

set mtTCP {0x3b 0xdf 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x80 0x18 0x7d 0x78 0x00 0x00 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

set goodTCP {0x3b 0xdf 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x80 0x18 0x7d 0x78 0x09 0x5f 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

# 0x09 0x5f is checksum

set p6 [packet::make IP version 4 hdrlen 5 tos 0 length 0x58 id 0x9a31 flag 2 \
    offset 0 ttl 0x40 protocol TCP checksum 0 source 192.168.9.4 dest 192.168.9.2 \
    options {} payload $mtTCP]

tcltest::test TCP-Checksum-1 {Checksum for TCP/IP packet} {} \
    {$p6 getField packet} \
    $goodIP


set p7 [packet::make TCP source 0x3bdf dest 0xa33f seq {0x8519 0x74c8} ack_seq 0x70142b97 \
     hlen 0x8 res 0 urg 0 ack 1 psh 1 rst 0 syn 0 fin 0 window 0x7d78 checksum 0\
     urg_ptr 0 option {} payload {0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}]

tcltest::test TCP-Make-Packet {Create a TCP Packet} {} \
     {$p7 getField packet} \
     $mtTCP


set modTCP {0x12 0x34 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x80 0x18 0x7d 0x78 0x00 0x00 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

    
# $p7 setField source 0x1234
# $p7 genPacket
$p7 replaceField source 0x1234

tcltest::test TCP-Replace-2-Bytes {Create a TCP Packet} {} \
     {$p7 getField packet} \
     $modTCP

set mod2TCP {0x12 0x34 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x80 0x08 0x7d 0x78 0x00 0x00 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

puts "START ACK REPLACE"
$p7 replaceField ack 0

tcltest::test TCP-Replace-bitfield-1 {Create a TCP Packet} {} \
     {$p7 getField packet} \
     $mod2TCP

set mod2TCP {0x12 0x34 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x80 0x09 0x7d 0x78 0x00 0x00 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

puts "START FIN REPLACE"
$p7 replaceField fin 1

tcltest::test TCP-Replace-bitfield-2 {Create a TCP Packet} {} \
     {$p7 getField packet} \
     $mod2TCP


set mod2TCP {0x12 0x34 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x40 0x09 0x7d 0x78 0x00 0x00 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

puts "START HLEN REPLACE"
$p7 replaceField hlen 4

tcltest::test TCP-Replace-multi-bitfield-3 {Create a TCP Packet} {} \
     {$p7 getField packet} \
     $mod2TCP


set ip_p7 {0x45 0x00 0x00 0x58 0x9a 0x31 0x40 0x00 0x40 0x06 0x0d 0x18 0xc0 0xa8 0x09 0x04\
0xc0 0xa8 0x09 0x02 0x12 0x34 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x40 0x09 0x7d 0x78 0x73 0x5c 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

set p8 [packet::make IP version 4 hdrlen 5 tos 0 length 0x58 id 0x9a31 flag 2 \
    offset 0 ttl 0x40 protocol TCP checksum 0 source 192.168.9.4 dest 192.168.9.2 \
    options {} payload $p7]

tcltest::test Build-IP-With-PacketID {Build an IP packet with a TCP packet name instead of data} {} \
    {$p8 getField packet} \
    $ip_p7
compareLists  [$p8 getField packet] $ip_p7

$p7 replaceField hlen 8
$p8 genPacket

set ip_p7a {0x45 0x00 0x00 0x58 0x9a 0x31 0x40 0x00 0x40 0x06 0x0d 0x18 0xc0 0xa8 0x09 0x04\
0xc0 0xa8 0x09 0x02 0x12 0x34 0xa3 0x3f 0x85 0x19 0x74 0xc8 0x70 0x14 0x2b 0x97\
0x80 0x09 0x7d 0x78 0x33 0x5c 0x00 0x00 0x01 0x01 0x08 0x0a 0x5c 0x2c 0xf3 0xf6\
0x18 0x7c 0x15 0x85 0x73 0x65 0x74 0x20 0x53 0x65 0x72 0x76 0x65 0x72 0x28 0x65\
0x74 0x68 0x30 0x2e 0x6f 0x6c 0x64 0x2e 0x72 0x63 0x76 0x2e 0x66 0x61 0x63 0x65\
0x29 0x20 0x65 0x74 0x68 0x30 0x0d 0x0a}

tcltest::test Modify-Field-1 {Replace a field in the child packet} {} \
    {$p8 getField packet} \
    $ip_p7a
compareLists  [$p8 getField packet] $ip_p7a

::tcltest::cleanupTests
exit

