set rev(udp.tcl) {$Header: /home/cvs/PacketMaker/udp.tcl,v 1.2 2002/12/31 02:40:01 clif Exp $}
#####################################################################
;# NAME:   udp.tcl
;# AUTHOR: Clif Flynt
;# DATE:   11/2002
;# DESC:   Procedures for manipulating UDP packets.
;#         
;#         
;#
;# PARAMETERS:  
;#
;#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 2002
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Noumena Corporation.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Noumena Corporation
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@noucorp.com
#
#####################################################################

package provide udp 1.0
# catch {source udp.tcl}

namespace eval udp {
    variable fields
    namespace export genPacket fillCheckSum 
    
    proc genPacket {} {
      upvar fieldArray dataArray

      variable fields
      variable lookup

      # UDP payloads may not be an odd number of bytes
      # Pad if necessary.

      if {[llength $dataArray(payload)] & 0x01} {
          lappend dataArray(payload) 0x00
      }

      
      # No checksum calculation done here - it needs info from
      #  IP header block, and gets done when the udp packet is
      #  inserted into the IP packet.

      set dataArray(packet) \
          [packet::MakePayload $fields lookup dataArray]
      return $dataArray(packet)
    }
    
    proc fillCheckSum {data {prefix {}} } {
      variable fields
      
      if {![string match [info procs ::$data] ""]} {
          return [fillCheckSumName $data $prefix]
      }
      
      set byteStream "$prefix $data"

      if {[llength $byteStream] & 0x01} {lappend byteStream 0x00}

      set p [lsearch $fields checksum]
      set pos [expr ([lindex $fields [incr p] ] / 8)]
      set len [expr ([lindex $fields [incr p]] / 8) -1]
puts "$fields P: $p POS: $pos LEN: $len"
      set checksum [packet::CheckSum $byteStream]

      set packet [eval lreplace [list $data] $pos [expr $pos + $len] $checksum]
puts "SUM: $checksum \nBYT: $byteStream\nPACKET: $packet"
      return $packet
    }

    proc fillCheckSumName {name prefix} {
        set data [$name getField packet]

        set byteStream "$prefix $data"
        if {[llength $byteStream] & 0x01} {lappend byteStream 0x00}
	
        set checksum [packet::CheckSum $byteStream]

	$name replaceField checksum $checksum
        return $name
    }
    
}



